/***************************************************************************
                          BitOp.h  -  description
                             -------------------
    begin                : Thu Sep 6 2001
    copyright            : (C) 2001 by
    email                :
 ***************************************************************************/
namespace rwil {
  namespace BitOp {

    /// Get the value of a single bit in data.
    /**
      * Returns a 0 or one depending on the value of a specific bit
      * @param data The memory buffer to examine
      * @param bit The number of the specific bit to look at
      * @return The value of the bit, 0 or 1
      */
    int GetBit(char *data, int bit);

    /// Gets the value of up to 32 bits
    /**
      * Returns an integer with the first numbits of it set to the desired bits from the data
      * @param data The memory buffer to examine
      * @param startbit The number of the first bit to examine
      * @param numbits The number of bits of data to extract max 32
      * @return The bit data
      */
    int GetBits(char *data, int startbit, int numbits);

    /// Set the value of a specific bit
    /**
      * Sets the value of a specific bit to val. val must be 0 or 1
      * @param data The memory buffer to modify
      * @param bit The number of the specific bit to modify
      * @val The new value of the bit, 0 or 1
      */
    void SetBit(char *data, int bit, int val);

    /// Sets the value of up to 32 bits at a time
    /**
      * Sets the value of up to 32 consecutive bits at a time.
      * @param data The memory buffer to modify.
      * @param startbit The identifier of the first bit to modify
      * @param numbits The number of bits to modify, max 32
      * @param val An integer representing a bit vector that is the new bit values.
      */
    void SetBits(char *data, int startbit, int numbits, int val);

    /// Print a series of bits
    /**
      * Print to stdout a series of one's and zeroes starting with the lsb.
      * @param data The memory buffer to display
      * @param bits The number of bits to display from data.
      */
    void PrintBits(char *data, int bits);

  }
}
