#include "Device.h"
namespace rwil {

  Device::Device(const std::string &name, MessageList& output) : m_output(output),  m_name(name), m_retries(2), m_currentRetry(0) {}

  Device::~Device()
  {
  }
  const std::string &Device::GetName() const
  {
    return m_name;
  }

  void Device::AddToQueue(const Message &msg)
  {
    //assert that the message is associated with this device
    assert(msg.GetDevice() == GetName());
	
    m_queue.push(msg);
  }

  //Execute the top message in the queue
  //Return true if anything is attempted
  bool Device::Tick()
  {
    if(m_queue.empty()) return false;

    if(m_queue.front().GetCommand() == "SetRetries")
      {
	try
	  {
	    m_retries = StringToInt(m_queue.front().GetParam("retries"));
	  }
	catch(const char * e)
	  {
	    cout << e << endl;
	  }
	m_queue.pop();
	return true;
      }
    else
      {
	ExecuteReturns::Returns retval;
	try
	  {
	    retval = ExecuteMessage(m_queue.front());
	  }
	catch (const char *e)
	  {
	    m_output.Add(EncapsulatingMessage(e, m_queue.front()));

	    m_queue.pop();
	    m_currentRetry = 0;
	    return true;
	  }
	catch (...)
	  {
	    m_output.Add(EncapsulatingMessage("Failed", m_queue.front()));

	    m_queue.pop();
	    m_currentRetry = 0;
	    return true;
	  }
	if(retval==ExecuteReturns::DontRetry)
	  {
	    m_queue.pop();
	    m_currentRetry = 0;
	  }
	else 
	  {
	    m_currentRetry++;
	    if(m_currentRetry>(m_retries+1))
	      {
		m_output.Add(FailedAfterRetryingMessage(m_queue.front()));
		m_queue.pop();
		m_currentRetry = 0;
	      }
	  }
      }
    return true;
  }
}
