
#include <time.h>
#include "Utility.h"
#include "Message.h"

namespace rwil {
  Message::Message(std::string deviceName, std::string command) : m_when(time(0))
  {
    m_parameters["device"] = deviceName;
    m_parameters["command"] = command;
  }

  Message::Message(std::string deviceName, std::string command, time_t when) : m_when(when)
  {
    m_parameters["device"] = deviceName;
    m_parameters["command"] = command;
  }

  Message::Message(std::map<std::string,std::string> params) : m_parameters(params), m_when(time(0))
  {
  }

  Message::Message(std::map<std::string,std::string> params, time_t when) : m_parameters(params), m_when(when)
  {
  }

  Message::Message(const Message &msg) : m_parameters(msg.m_parameters), m_when(msg.m_when) {}


  const std::string& Message::GetDevice() const { return m_parameters["device"]; }
  const std::string& Message::GetCommand() const { return m_parameters["command"]; }

  bool Message::operator<(const Message& rhs)
  {
    return this->m_when < rhs.m_when;
  }

  void Message::SetParam(std::string n, std::string param) { m_parameters[n] = param; }
  std::string Message::GetParam(std::string n) const { return m_parameters[n]; }

  time_t Message::GetWhen() const { return m_when; }
  const std::map<std::string,std::string>& Message::GetMap() const { return m_parameters; }


  EncapsulatingMessage::EncapsulatingMessage(std::string encapsulateCommand, const Message &oldMessage) : Message(oldMessage.GetMap())
  {
    this->SetParam("message", encapsulateCommand);
  }
  EncapsulatingMessage::EncapsulatingMessage(std::string encapsulateCommand, const Message & oldMessage, time_t when) : Message(oldMessage.GetMap(), when)
  {
    this->SetParam("message", encapsulateCommand);
  }

  FailedAfterRetryingMessage::FailedAfterRetryingMessage(const Message & oldMessage) : EncapsulatingMessage("Failed after retrying message", oldMessage) {}

  SetRetriesMessage::SetRetriesMessage(std::string deviceName, unsigned retries) : Message(deviceName, "SetRetries")
  {
    this->SetParam("retries", IntToString(retries));
  }
}
