#include "MessageList.h"
#include <stdio.h>
namespace rwil {
  void MessageList::Add(const Message& msg)
  {
    list<Message> lst;

    lst.push_back(msg);
    Add(lst);
  }
  void MessageList::Add(list<Message>& messages)
  {
    if(m_addcallback)
      m_addcallback(messages);
    ScopedMutex scope(m_op);
    messages.sort();
    m_messages.merge(messages);
    m_count.Signal();
  }

  void MessageList::SetAddCallback(rwil_MessageList_AddCallback_t callback)
  {
    m_addcallback = callback;
  }
  list<Message *> MessageList::Query(time_t start, time_t end)
  {
    ScopedMutex scope(m_op);
    list<Message>::iterator iter;
    list<Message *> retval;

    iter = m_messages.begin();
    while(iter != m_messages.end())
      {
	time_t when = (*iter).GetWhen();
	if((when >= start) && ( when <=end))
	  {
	    retval.push_back(&(*iter));
	  }

	iter++;
      }
    return retval;
  }

  list<Message *> MessageList::Query(std::string paramname, std::string value)
  {
    ScopedMutex scope(m_op);
    list<Message>::iterator iter;
    list<Message *> retval;

    iter = m_messages.begin();

    while(iter != m_messages.end())
      {
	if((*iter).GetParam(paramname) == value)
	  {
	    retval.push_back(&(*iter));
	  }
	iter++;
      }
    return retval;
  }


  void MessageList::Destroy(list<Message*> messages)
  {
    ScopedMutex scope(m_op);
    list<Message *>::iterator piter;
    list<Message>::iterator miter;
    piter = messages.begin();
    while(piter != messages.end())
      {
	miter = m_messages.begin();
	while(miter != m_messages.end())
	  {
	    if(&(*miter) == *piter)
	      {
		m_messages.erase(miter);
		break;
	      }
	  }
	piter++;
      }
  }

  void MessageList::Lock()
  {
    m_lock.Wait();
  }
  void MessageList::Release()
  {
    m_lock.Release();
  }

  bool MessageList::Wait(int milli)
  {
    return m_count.Wait(milli);
  }

  void MessageList::Wait()
  {
    return m_count.Wait();
  }
}
