#include "X10Device.h"
namespace rwil {
  ExecuteReturns::Returns X10Device::ExecuteMessage(const Message & msg)
  {
    char house, number;
    char temp[2];
    temp[1] = '\0';
    temp[0]= m_house;
    house = GetRealHouse(temp);
    number = GetRealNumber(m_number);
    if(msg.GetCommand()=="dim")
      {
	int percent = StringToInt(msg.GetParam("percent"));
	assert((percent >= 0) && (percent <= 100));
	if(!SupportsDim()) assert(0);
	X10::send_Dim_X10(house, number, percent);
	int real_percent = X10::get_status_X10(house, number);
	if((real_percent < 0) || (abs(percent-real_percent) > 3))
	  {
	    return ExecuteReturns::PleaseRetry;
	  }
	brightness = real_percent;
      }
    if(msg.GetCommand()=="off")
      {

	if(!SupportsDim()) 
	  {
	    X10::send_X10(house, number, 'G');
	  }
	else
	  {
	    X10::send_Dim_X10(house, number, 0);
	  }
	int real_percent = X10::get_status_X10(house, number);
	if((real_percent < 0) || (abs(real_percent) > 3))
	  {
	    return ExecuteReturns::PleaseRetry;
	  }
	brightness = 0;
      }
    if(msg.GetCommand()=="on")
      {
	if(!SupportsDim()) 
	  {
	    X10::send_X10(house, number, 'E');
	  }
	else
	  {
	    X10::send_Dim_X10(house, number, 100);
	  }
	int real_percent = X10::get_status_X10(house, number);
	if((real_percent < 0) || (abs(100-real_percent) > 3))
	  {
	    return ExecuteReturns::PleaseRetry;
	  }
	brightness = 100;
      }

    m_output.Add(EncapsulatingMessage("success", msg));
    return ExecuteReturns::DontRetry;
  }

  char X10Device::GetRealHouse(std::string house) const
  {
    char chouse = house[0];
    switch(chouse)
      {
      case 'a' : case 'A' : return 'F';
      case 'b' : case 'B' : return 'N';
      case 'c' : case 'C' : return 'B';
      case 'd' : case 'D' : return 'J';
      case 'e' : case 'E' : return 'A';
      case 'f' : case 'F' : return 'I';
      case 'g' : case 'G' : return 'E';
      case 'h' : case 'H' : return 'M';
      case 'i' : case 'I' : return 'G';
      case 'j' : case 'J' : return 'O';
      case 'k' : case 'K' : return 'C';
      case 'l' : case 'L' : return 'K';
      case 'm' : case 'M' : return '@';
      case 'n' : case 'N' : return 'H';
      case 'o' : case 'O' : return 'D';
      case 'p' : case 'P' : return 'L';
      }
    assert(0); //should never get here
  }

  char X10Device::GetRealNumber(int unitnumber) const
  {
    switch(unitnumber)
      {
      case 1 : return 'L';
      case 2 : return '\\';
      case 3 : return 'D';
      case 4 : return 'T';
      case 5 : return 'B';
      case 6 : return 'R';
      case 7 : return 'J';
      case 8 : return 'Z';
      case 9 : return 'N';
      case 10 : return '^';
      case 11 : return 'F';
      case 12 : return 'V';
      case 13 : return '@';
      case 14 : return 'P';
      case 15 : return 'H';
      case 16 : return 'X';
      }
    assert(0); //should never get here	
  }

  void X10Device::SetHouse(std::string house) { m_house = house[0]; }
  void X10Device::SetNumber(int number) { m_number = number; }

  bool X10Device::SupportsDim() {return true; }
  std::string X10Device::GetStatus() const { return IntToString(brightness); }
  X10Device::X10Device(const std::string &name, MessageList& output, char house, int number) : Device(name, output), m_house(house), m_number(number) {}
}
