#include "Device.h"
#include "Utility.h"

#ifndef X10DEVICE_HEADER
#define X10DEVICE_HEADER
#include "X10Interface.h"
namespace rwil {
  /// This is a representation of a single X10 device in the framework
  class X10Device : public Device
    {
    public:
      /// Support function that returns true if the device supports dimming. CURRENTLY always returns true. Needs to be fixed
      bool SupportsDim();
      /// Returns a string containing the cached value of how much power is being applied to the device 0-100
      std::string GetStatus() const;
      /// Constructs an X10 device with the specified name attached to the specified x10 address
      X10Device(const std::string &name, MessageList& output, char house, int number);
      /// Function to execute a message. This is called automatically
      virtual ExecuteReturns::Returns ExecuteMessage(const Message & msg);

      /// Sets the house letter a-p or A-P
      void SetHouse(std::string house);
      
      /// Sets the unit number 1-16
      void SetNumber(int number);
    protected:  
      /// Given a house letter returns what it is in Powerlinc terms
      char GetRealHouse(std::string house) const;

      /// Given a unit number returns what it is in Powerlinc terms
      char GetRealNumber(int unitnumber) const;

      /// The house letter
      char m_house;
      /// The unit number
      int m_number;
      /// The cached value of the percent bright the device is. 0-100
      int brightness;
    };
}

#endif
