#include "rwil.h"
namespace rwil {
  RWIL::RWIL() : m_die(false) {}
  void RWIL::AddDevice(Device *dev)
  {
    ScopedMutex scpm(m_mutex);
    m_devices.push_back(dev);
  }
       
  Device *RWIL::GetDevice(std::string name)
  {

    ScopedMutex scpm(m_mutex);

    std::list<Device*>::iterator iter = m_devices.begin();

    while((iter != m_devices.end()) && !m_devices.empty())
      {
	if((*iter)->GetName() == name)
	  {
	    return *iter;
	  }
	iter++;
      }
    return NULL;
  }

  void RWIL::RemoveDevice(std::string name)
  {

    ScopedMutex scpm(m_mutex);	
    std::list<Device*>::iterator iter = m_devices.begin();
    while(iter != m_devices.end())
      {
	if((*iter)->GetName() == name)
	  {
			
	    m_devices.erase(iter);
	    break;
	  }
	iter++;
      }
  }

  void RWIL::Function()
  {
    bool temp;
		
    while(m_mutex.Wait(), temp = m_die, m_mutex.Release(), temp == false)
      {
	input.Lock();
	std::list<Message*> moving(input.Query(0, time(NULL)));
	std::list<Message*>::iterator movingIter = moving.begin();
	std::list<Message*> removed;
			
	//Move all messages into the appropriate devices
	while(movingIter != moving.end())
	  {
	    std::list<Device*>::iterator deviceIter = m_devices.begin();
	    while(deviceIter != m_devices.end())
	      {
		if((*deviceIter)->GetName() == ((*movingIter)->GetDevice()))
		  {
		    (*deviceIter)->AddToQueue(**movingIter);
		    removed.push_back(*movingIter);
		  }
		deviceIter++;
	      }
	    movingIter++;
	  }

	input.Destroy(removed);
	input.Release();
	//Iterate through all the devices and tell them to work their magic
	std::list<Device*>::iterator deviceIter = m_devices.begin();
	while(deviceIter != m_devices.end())
	  {
	    (*deviceIter)->Tick();
	    deviceIter++;
	  }			
      }
    m_die = false;
    return;
  }
	
  void RWIL::Kill()
  {
    ScopedMutex scpm(m_mutex);
    m_die = true;
  }
}
