#include "Device.h"

namespace rwil {
  /// The object that represents the rwil. To start the system, call Start().
  class RWIL : public Thread
    {
    public:
      /// Constructs a RWIL object
      RWIL();
      /// Represents the input messages
      MessageList input;
      /// Represents the output messages
      MessageList output;

      /// Add a device to this instance of the implementation
      /**
        * Device must have unique name
        * @param Pointer to an instance of a class that derives from Device
        */
      void AddDevice(Device *dev);
      /// Get a device from its name
      /**
        * @param Name of device to find in system
        * @return A pointer to the device or, NULL if the device does not exist
        */
      Device *GetDevice(std::string name);
      /// Remove a device by name
      /**
        * If the device does not exist, this call does nothing.
        * @param name Name of device to remove.
        */
      void RemoveDevice(std::string name);
      /// The overloaded function that implements the RWIL functionality.
      virtual void Function();
      /// Kill the thread.
      /**
        * This returns immediately, but thread actually takes a long time to die.
        * It is recommended to Join this thread to determine when done.
        */
      void Kill();
    private:
      /// The list of devices
      std::list<Device *> m_devices;
      /// The mutex that maintains thread safety
      Mutex m_mutex;
      /// Indicates that it is time for the thread to die.
      bool m_die;
    };
}
